package com.sun.xml.internal.ws.server.sei;

import com.sun.xml.internal.ws.api.model.JavaMethod;
import com.sun.xml.internal.ws.api.message.Packet;
import javax.xml.ws.ProtocolException;
import com.sun.xml.internal.ws.fault.SOAPFaultBuilder;
import java.util.logging.Level;
import com.sun.xml.internal.ws.wsdl.DispatchException;
import java.lang.reflect.InvocationTargetException;
import com.oracle.webservices.internal.api.databinding.JavaCallInfo;
import javax.xml.stream.XMLStreamException;
import javax.xml.bind.JAXBException;
import javax.xml.ws.WebServiceException;
import com.sun.xml.internal.ws.api.message.Message;
import java.util.Iterator;
import javax.jws.WebParam;
import com.sun.xml.internal.ws.model.WrapperParameter;
import com.sun.xml.internal.ws.model.ParameterImpl;
import java.util.List;
import java.util.ArrayList;
import com.sun.xml.internal.ws.api.WSBinding;
import java.util.logging.Logger;
import com.sun.xml.internal.ws.api.message.MessageContextFactory;
import com.sun.xml.internal.ws.model.JavaMethodImpl;
import java.lang.reflect.Method;
import com.sun.xml.internal.ws.api.SOAPVersion;
import com.sun.xml.internal.ws.api.databinding.EndpointCallBridge;

public final class TieHandler implements EndpointCallBridge
{
    private final SOAPVersion soapVersion;
    private final Method method;
    private final int noOfArgs;
    private final JavaMethodImpl javaMethodModel;
    private final Boolean isOneWay;
    private final EndpointArgumentsBuilder argumentsBuilder;
    private final EndpointResponseMessageBuilder bodyBuilder;
    private final MessageFiller[] outFillers;
    protected MessageContextFactory packetFactory;
    private static final Logger LOGGER;
    
    public TieHandler(JavaMethodImpl method, WSBinding binding, MessageContextFactory mcf) {
        this.soapVersion = binding.getSOAPVersion();
        this.method = method.getMethod();
        this.javaMethodModel = method;
        this.argumentsBuilder = this.createArgumentsBuilder();
        List<MessageFiller> fillers = new ArrayList();
        this.bodyBuilder = this.createResponseMessageBuilder(fillers);
        this.outFillers = (MessageFiller[])fillers.toArray(new MessageFiller[fillers.size()]);
        this.isOneWay = Boolean.valueOf(method.getMEP().isOneWay());
        this.noOfArgs = this.method.getParameterTypes().length;
        this.packetFactory = mcf;
    }
    
    private EndpointArgumentsBuilder createArgumentsBuilder() {
        List<ParameterImpl> rp = this.javaMethodModel.getRequestParameters();
        List<EndpointArgumentsBuilder> builders = new ArrayList();
        for (ParameterImpl param : rp) {
            EndpointValueSetter setter = EndpointValueSetter.get(param);
            switch (param.getInBinding().kind) {
                case BODY: {
                    if (param.isWrapperStyle()) {
                        if (param.getParent().getBinding().isRpcLit()) {
                            builders.add(new EndpointArgumentsBuilder.RpcLit((WrapperParameter)param));
                            continue;
                        }
                        builders.add(new EndpointArgumentsBuilder.DocLit((WrapperParameter)param, WebParam.Mode.OUT));
                        continue;
                    }
                    else {
                        builders.add(new EndpointArgumentsBuilder.Body(param.getXMLBridge(), setter));
                        continue;
                    }
                }
                case HEADER: {
                    builders.add(new EndpointArgumentsBuilder.Header(this.soapVersion, param, setter));
                    continue;
                }
                case ATTACHMENT: {
                    builders.add(EndpointArgumentsBuilder.AttachmentBuilder.createAttachmentBuilder(param, setter));
                    continue;
                }
                case UNBOUND: {
                    builders.add(new EndpointArgumentsBuilder.NullSetter(setter, EndpointArgumentsBuilder.getVMUninitializedValue(param.getTypeInfo().type)));
                    continue;
                }
                default: {
                    throw new AssertionError();
                }
            }
        }
        List<ParameterImpl> resp = this.javaMethodModel.getResponseParameters();
        for (ParameterImpl param2 : resp) {
            if (param2.isWrapperStyle()) {
                WrapperParameter wp = (WrapperParameter)param2;
                List<ParameterImpl> children = wp.getWrapperChildren();
                for (ParameterImpl p : children) {
                    if (p.isOUT() && p.getIndex() != -1) {
                        EndpointValueSetter setter3 = EndpointValueSetter.get(p);
                        builders.add(new EndpointArgumentsBuilder.NullSetter(setter3, null));
                    }
                    else {
                        continue;
                    }
                }
            }
            else if (param2.isOUT() && param2.getIndex() != -1) {
                EndpointValueSetter setter2 = EndpointValueSetter.get(param2);
                builders.add(new EndpointArgumentsBuilder.NullSetter(setter2, null));
            }
            else {
                continue;
            }
        }
        EndpointArgumentsBuilder argsBuilder;
        switch (builders.size()) {
            case 0: {
                argsBuilder = EndpointArgumentsBuilder.NONE;
                break;
            }
            case 1: {
                argsBuilder = (EndpointArgumentsBuilder)builders.get(0);
                break;
            }
            default: {
                argsBuilder = new EndpointArgumentsBuilder.Composite(builders);
                break;
            }
        }
        return argsBuilder;
    }
    
    private EndpointResponseMessageBuilder createResponseMessageBuilder(List<MessageFiller> fillers) {
        EndpointResponseMessageBuilder tmpBodyBuilder = null;
        List<ParameterImpl> rp = this.javaMethodModel.getResponseParameters();
        for (ParameterImpl param : rp) {
            ValueGetter getter = ValueGetter.get(param);
            switch (param.getOutBinding().kind) {
                case BODY: {
                    if (param.isWrapperStyle()) {
                        if (param.getParent().getBinding().isRpcLit()) {
                            tmpBodyBuilder = new EndpointResponseMessageBuilder.RpcLit((WrapperParameter)param, this.soapVersion);
                            continue;
                        }
                        tmpBodyBuilder = new EndpointResponseMessageBuilder.DocLit((WrapperParameter)param, this.soapVersion);
                        continue;
                    }
                    else {
                        tmpBodyBuilder = new EndpointResponseMessageBuilder.Bare(param, this.soapVersion);
                        continue;
                    }
                }
                case HEADER: {
                    fillers.add(new MessageFiller.Header(param.getIndex(), param.getXMLBridge(), getter));
                    continue;
                }
                case ATTACHMENT: {
                    fillers.add(MessageFiller.AttachmentFiller.createAttachmentFiller(param, getter));
                    break;
                }
                case UNBOUND: {
                    continue;
                }
                default: {
                    throw new AssertionError();
                }
            }
        }
        if (tmpBodyBuilder == null) {
            switch (this.soapVersion) {
                case SOAP_11: {
                    tmpBodyBuilder = EndpointResponseMessageBuilder.EMPTY_SOAP11;
                    break;
                }
                case SOAP_12: {
                    tmpBodyBuilder = EndpointResponseMessageBuilder.EMPTY_SOAP12;
                    break;
                }
                default: {
                    throw new AssertionError();
                }
            }
        }
        return tmpBodyBuilder;
    }
    
    public Object[] readRequest(Message reqMsg) {
        Object[] args = new Object[this.noOfArgs];
        try {
            this.argumentsBuilder.readRequest(reqMsg, args);
        }
        catch (JAXBException e) {
            throw new WebServiceException(e);
        }
        catch (XMLStreamException e2) {
            throw new WebServiceException(e2);
        }
        return args;
    }
    
    public Message createResponse(JavaCallInfo call) {
        Message responseMessage;
        if (call.getException() == null) {
            responseMessage = (((boolean)this.isOneWay) ? null : this.createResponseMessage(call.getParameters(), call.getReturnValue()));
        }
        else {
            Throwable e = call.getException();
            Throwable serviceException = this.getServiceException(e);
            if (e instanceof InvocationTargetException || serviceException != null) {
                if (serviceException != null) {
                    TieHandler.LOGGER.log(Level.FINE, serviceException.getMessage(), serviceException);
                    responseMessage = SOAPFaultBuilder.createSOAPFaultMessage(this.soapVersion, this.javaMethodModel.getCheckedException(serviceException.getClass()), serviceException);
                }
                else {
                    Throwable cause = e.getCause();
                    if ((cause instanceof ProtocolException)) {
                        TieHandler.LOGGER.log(Level.FINE, cause.getMessage(), cause);
                    }
                    else {
                        TieHandler.LOGGER.log(Level.SEVERE, cause.getMessage(), cause);
                    }
                    responseMessage = SOAPFaultBuilder.createSOAPFaultMessage(this.soapVersion, null, cause);
                }
            }
            else if ((e instanceof DispatchException)) {
                responseMessage = ((DispatchException)e).fault;
            }
            else {
                TieHandler.LOGGER.log(Level.SEVERE, e.getMessage(), e);
                responseMessage = SOAPFaultBuilder.createSOAPFaultMessage(this.soapVersion, null, e);
            }
        }
        return responseMessage;
    }
    
    Throwable getServiceException(Throwable throwable) {
        if (this.javaMethodModel.getCheckedException(throwable.getClass()) != null) {
            return throwable;
        }
        if (throwable.getCause() != null) {
            Throwable cause = throwable.getCause();
            if (this.javaMethodModel.getCheckedException(cause.getClass()) != null) {
                return cause;
            }
        }
        return null;
    }
    
    private Message createResponseMessage(Object[] args, Object returnValue) {
        Message msg = this.bodyBuilder.createMessage(args, returnValue);
        for (MessageFiller filler : this.outFillers) {
            filler.fillIn(args, returnValue, msg);
        }
        return msg;
    }
    
    public Method getMethod() {
        return this.method;
    }
    
    public JavaCallInfo deserializeRequest(Packet req) {
        com.sun.xml.internal.ws.api.databinding.JavaCallInfo call = new com.sun.xml.internal.ws.api.databinding.JavaCallInfo();
        call.setMethod(this.getMethod());
        Object[] args = this.readRequest(req.getMessage());
        call.setParameters(args);
        return call;
    }
    
    public Packet serializeResponse(JavaCallInfo call) {
        Message msg = this.createResponse(call);
        Packet p = (msg == null) ? ((Packet)this.packetFactory.createContext()) : ((Packet)this.packetFactory.createContext(msg));
        p.setState(Packet.State.ServerResponse);
        return p;
    }
    
    public JavaMethod getOperationModel() {
        return this.javaMethodModel;
    }
    
    static {
        LOGGER = Logger.getLogger(TieHandler.class.getName());
    }
}
